import datetime
from datetime import timedelta
import os
import jstyleson
import pytz
from google.cloud import pubsub_v1
from google.oauth2 import service_account

def cleanLastTask(now,SYM):
    
    expireTask = SYM.mongo[SYM.db+"_info"]['task'].aggregate([
        {
            "$match":
                {
                "$expr":
                    {
                        "$lt":
                            [
                            "$trigger.create",
                                {
                                "$dateSubtract":
                                    {
                                        "startDate": now,
                                        "unit": "second",
                                        "amount": "$trigger.timeout"
                                    }
                                }
                            ]
                    }
                }
        },
        {
            "$project": {
                "_id": 1,
            }
        }
        
    ])
    for task in expireTask:
        SYM.mongo[SYM.db+"_info"]['task'].delete_one({"_id":task['_id']})
    
    dbs = SYM.mongo.list_database_names()
    for db in dbs:
        if db[:7] == "___task":
            count = SYM.mongo[db]['000_trigger'].count_documents({})
            if count > 0:
                count = SYM.mongo[db]['000_trigger'].aggregate([
                    {
                        "$match":
                            {
                            "$expr":
                                {
                                    "$gt":
                                        [
                                        "$create",
                                            {
                                            "$dateSubtract":
                                                {
                                                    "startDate": now,
                                                    "unit": "second",
                                                    "amount": "$timeout"
                                                }
                                            }
                                        ]
                                }
                            }
                    },
                    {
                       "$count": "count"
                    }
                ])
                count = list(count)[0]['count']
                
            if count == 0:
                #print("drop database "+db)
                SYM.mongo.drop_database(db)
def shedule(now,SYM):
    paris = pytz.timezone('Europe/Paris') 
    now = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    for Workflow1  in os.listdir('./workflow'):
        if(Workflow1.find(".py") == -1):
            continue
        try:
            wfDev = SYM.workflow(Workflow1.replace(".py", ""))
            trigger =  wfDev.trigger({},SYM) 
            wj =  wfDev.SYM({},SYM) 
            wj.insert(0,{"$trigger" : trigger})
        except:
            continue
        if  trigger.get('shedule') is not None:
            trigger['shedule'] = int(trigger['shedule'])
            
            task = SYM.mongo[SYM.db+"_info"]['task'].count_documents(
                {
                    "trigger.name":Workflow1.replace(".py", ""),
                    "trigger.create" : {"$gt" : now  }
                })
            if task == 0:
                trigger['create'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2) + timedelta(seconds=trigger['shedule'])
                trigger['update'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2) + timedelta(seconds=trigger['shedule'])
                
                SYM.mongo[SYM.db+"_info"]["task"].insert_one({"workflow":wj,"trigger":trigger})

def executeTask(now,SYM):
    
    limitTotal = int(os.getenv('LIMIT')) - SYM.mongo[SYM.db+"_info"]['task'].count_documents({"run":1})
    limitTask = {}
    if limitTotal > 10:
        limitFind = 10
    else:
        limitFind = limitTotal
    alltask = SYM.mongo[SYM.db+"_info"]['task'].find({"trigger.create" : {"$lt" : now}}).limit(limitFind)
    execute = 0
    controller = SYM.controller("workflow")
    for task in alltask:
        if limitTask.get(task['trigger']['name']) is None:
            limitTask[task['trigger']['name']] = task['trigger']['thread']
        
        if limitTask[task['trigger']['name']] > 0 and limitTotal > 0:
            SYM.mongo[SYM.db+"_info"]['task'].update_one({"_id":task['_id']},{"$set":{"run":1}})
            
            result =  controller.run(
                {
                    "folder" : [],
                    "get" : [],
                    "body" : task['workflow'],
                },
                SYM
            ) 
            SYM.mongo[SYM.db+"_info"]['task'].delete_one({"_id":task['_id']})
            limitTask[task['trigger']['name']] -= 1
            limitTotal -= 1
            execute += 1
    if execute == 0:
        return 0
    else:
        return limitTotal

def run(request,SYM):
    paris = pytz.timezone('Europe/Paris') 
    now = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    #now = datetime.now()
    cleanLastTask(now,SYM)
    shedule(now,SYM)
    run = executeTask(now,SYM)
    if run > 0:
        if SYM.env == "local":
            controller = SYM.controller("runWorkflow")
            result =  controller.run(
                {
                    "folder" : [],
                    "get" : [],
                    "body" : [],
                },
                SYM
            )
        else:
            publisher = pubsub_v1.PublisherClient(credentials=credentials)
            topic_path = publisher.topic_path("scrapyomama-335508", "task")
            data = json.dumps({
                    "folder" : ["runWorkflow"],
                    "get" : [],
                    "body" : [],
                }, default=str)
            data = data.encode("utf-8")
            future = publisher.publish(topic_path, data=data)
            #print(future.result())
    
    return ([])



import uuid
import pytz
import datetime
from datetime import  date, timedelta
import json
import traceback
import jstyleson

def parseFolder(request,SYM):
    executeFolder = {
        "insert" : {

        },
        "append" : {

        }
    }
    key = 1
    append=0
    #print(request['folder'])
    if len(request['folder']) > 0:
        request['folder'].insert(0,"webhook")
    for folder in request['folder']:
        if folder == "append":
            append = 1
        elif append == 0:
            if key == 1:
                lastkey = folder
                key = 0
            else:
                executeFolder['insert'][lastkey] = folder
                key = 1
        elif append == 1:
            if key == 1:
                lastkey = folder
                key = 0
            else:
                executeFolder['append'][lastkey] = folder
                append=0
                key = 1

    return(executeFolder)

def workflowfile(execute,fileExe,request,workflow,SYM):
        wfDev = SYM.workflow(fileExe)
        wf =  wfDev.SYM({},SYM)
        try:
            trigger =  wfDev.trigger({},SYM) 
            wf.insert(0,{"$trigger" : trigger}) 
        except:
            print("no trigger")
        i = 0
        if wf[0].get("$trigger") is not None:
            workflow.insert(0, wf[0])
            workflow[0]["$trigger"]['name'] = fileExe
            wf.pop(0)
            i = 1
        if execute == "insert":
            wf.reverse()
            for task in wf:
                workflow.insert(i, task)
        else:
            for task in wf:
                workflow.append(task)
        #request['webhook'] = request['body']
        return i
        
        
def preparWorkflow(executeFolder,workflow,request,SYM):
    i = 0
    wh = 0
    for folder in request['folder']:
        if folder == "webhook":
            wh = 1
    if wh == 0:
        print("no webhook")
        for body in request['body']:
            workflow.append(body)
        if len(workflow) == 0:
            i = i
        elif workflow[0].get("$trigger") is not None:
            i += 1

    for insert in executeFolder['insert']:
        if insert == "webhook":
            i += workflowfile("insert",executeFolder['insert'][insert],request,workflow,SYM)
            #print(request['body'])
            workflow.insert(i,{
                "$webhook" : request['body']
            })
            i += 1
        else:
            workflow.insert(i,{
                "$readCollection" : {
                    "collection" : executeFolder['insert'][insert],
                    "database" : insert,
                }
            })
            i += 1
    for append in executeFolder['append']:
        if append == "webhook":
            workflowfile("append",executeFolder['append'][append],request,workflow,SYM)
        else:
            workflow.append({
                "$readCollection" : {
                    "collection" : executeFolder['append'][append],
                    "database" : append,
                }
            })    
    return(workflow)

def createtrigger(workflow,request,SYM):
    if workflow[0].get("$trigger") is None:
        workflow.insert(0,{
            "$trigger" : request['get']
        }) 
    else:
        for get in request['get']:
            workflow[0]['$trigger'][get] = request['get'][get]

def verifTrigger(trigger,SYM):
    if trigger.get("limit") is  None:
        trigger['limit'] = 100
    if trigger.get("thread") is  None:
        trigger['thread'] = 1
    if trigger.get("offset") is  None:
        trigger['offset'] = 0
    if trigger.get("timeout") is  None:
        trigger['timeout'] = 3600
    if trigger.get("taskId") is  None:
        trigger['taskId'] = str(uuid.uuid4())
    trigger['limit'] = int(trigger['limit'])
    trigger['thread'] = int(trigger['thread'])
    trigger['offset'] = int(trigger['offset'])
    trigger['timeout'] = int(trigger['timeout'])

def startWorkflow(workflow,trigger,request,SYM):
    paris = pytz.timezone('Europe/Paris') 
    trigger['n'] = 0
    
    trigger['create'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    trigger['update'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    #result = workflow[0]
    collection = str(trigger['n']).zfill(3)+"_trigger"
    
    SYM.mongo["___task_"+trigger['name']+"_"+trigger['taskId']][collection].drop()
    
    SYM.mongo["___task_"+trigger['name']+"_"+trigger['taskId']][collection].insert_one(dict(trigger))
    #trigger['n'] += 1

def runNode(database,nodeApp,node,last_query,trigger,workflow,request,SYM):
    app = SYM.app(nodeApp)
    i = 0
    nodes = []
    for w in workflow:
        if i < trigger['n']:
            
            wNodeApp = list(w.keys())[0].replace("$","")
            wNode = w["$"+wNodeApp]
            collection = str(i).zfill(3)+"_"+wNodeApp
            if wNodeApp == "readCollection":
                wQuery = SYM.mongo[SYM.db+"_"+wNode['database']][wNode['collection']]
            else:
                wQuery = SYM.mongo[database][collection]
            nodes.append({
                "get" : wNode,
                "request" : wQuery,
                "app": wNodeApp
            })
            
            i += 1

    body = {
        "trigger" : trigger,
        "workflow" : workflow,
        "nodes" : nodes,
        "node" : {
            "get" : node["$"+nodeApp],
		    "request" : last_query,
		    "app": nodeApp
        }
    }
    try:
        result =  app.run(
            {
                "get" : request['get'],
                "body" : body,
                "folder" : request['folder'],
            },
            SYM
        )
    except:
        error = str(traceback.format_exc()).split("\n")
        #error = traceback.format_exc()
        #error = traceback.format_list(error)
        result = {"error" : error}
        SYM.mongo[SYM.db+"_error"][trigger['name']].insert_one({"workflow":workflow,"trigger":trigger,"result":result})
        trigger['n'] = len(workflow)-1

    return result

def saveResult(result,database,collection,SYM):
    SYM.mongo[database][collection].drop()
    if isinstance(result, list):
        insertResult = []
        for i in result:
            if i.get('_id') is not None:
                del i['_id']
            insertResult.append(i)
            if len(insertResult) == 100:
                SYM.mongo[database][collection].insert_many(insertResult)
                insertResult = []
        if len(insertResult) > 0:
            SYM.mongo[database][collection].insert_many(insertResult)
            insertResult = []
    else:
        if(result.get("async") is None):
            if result.get('_id') is not None:
                del result['_id']
            #print(result)
            SYM.mongo[database][collection].insert_many([dict(result)])

def nextStep(node,current_query,trigger,workflow,database,collection,result,request,SYM):
    if(trigger['n'] == len(workflow)):
        if trigger.get('save') is not None:
            r = current_query.aggregate([
                { "$out": { "coll": trigger['name'], "db": SYM.db+"_SYM" } }
            ])
        trigger['n'] = -1    
        
        trigger['async'] = 1
        r = current_query.find({}).skip(trigger['offset']).limit(trigger['limit'] )
        ret = json.loads(json.dumps({
            "n" : len(workflow)-1,
            "end" : 1,
            "results" : list(r),
            "node" : node,
            "trigger" : trigger,
            "workflow" : workflow
        }, default=str))
        SYM.mongo.drop_database(database)
    else:
        if trigger.get("backgroud") is None:
            controller = SYM.controller("workflow")
            result =  controller.run(
                {
                    "folder" : request['folder'],
                    "get" : request['get'],
                    "body" : workflow,
                },
                SYM
            ) 
            ret = json.loads(json.dumps(result, default=str))
        elif isinstance(result, list):
            SYM.mongo[SYM.db+"_info"]["task"].insert_one({"workflow":workflow,"trigger":trigger})
            trigger['async'] = 0
            #ret = json.loads(json.dumps(workflow, default=str))
        elif result.get("async") is None:
            SYM.mongo[SYM.db+"_info"]["task"].insert_one({"workflow":workflow,"trigger":trigger})
            trigger['async'] = 0
            #ret = json.loads(json.dumps(workflow, default=str))
        else:
            trigger['async'] = 1
            #ret = json.loads(json.dumps(workflow, default=str))
        if trigger.get("backgroud") is not None:
            r = current_query.find({}).skip(trigger['offset']).limit(trigger['limit'] )
            ret = json.loads(json.dumps({
                "n" : trigger['n']-1,
                "results" : list(r),
                "node" : node,
                "trigger" : trigger,
                "workflow" : workflow
            }, default=str))
    return ret

def affichResult(request,SYM):
    if request['get'].get("offset") is None:
        request['get']['offset'] = 0
    if request['get'].get("limit") is None:
        request['get']['limit'] = 500
    if request['get'].get("find") is None:
        request['get']['find'] = {}
    else:
        request['get']['find'] = jstyleson.loads(request['get']['find'])
    request['get']['offset'] = int(request['get']['offset'])  
    request['get']['limit'] = int(request['get']['limit'])
    #print(request['get']['find'])       
    result = SYM.mongo[SYM.db+"_SYM"][request['folder'][0]].find(request['get']['find'] ).skip(request['get']['offset']).limit(request['get']['limit'] )
    return json.loads(json.dumps({"results" : list(result)}, default=str))

def run(request,SYM):
    paris = pytz.timezone('Europe/Paris') 

    workflow = []
    nexist = 0
    if request['get'].get('taskId') is not None:
        now = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
        alltask = SYM.mongo[SYM.db+"_info"]['task'].find({"trigger.taskId" : request['get']['taskId'], "trigger.create" : {"$lt" : now}}).sort("trigger.create", -1).limit(1)
        for task in alltask:
            request['body'] = task['workflow']
            SYM.mongo[SYM.db+"_info"]['task'].delete_one({"_id":task['_id']})

    if isinstance(request['body'], list):
        if len(request['body']) > 0:
            if request['body'][0].get('$trigger') is not None:
               if request['body'][0]['$trigger'].get('n') is not None:    
                    nexist = 1
    if nexist == 0:                
        executeFolder = parseFolder(request,SYM)
        preparWorkflow(executeFolder,workflow,request,SYM)
    else:
        workflow = request['body']

    #return(workflow)
    createtrigger(workflow,request,SYM)
    trigger = workflow[0]['$trigger']
    verifTrigger(trigger,SYM)
    
    if trigger.get("n") is None:
        startWorkflow(workflow,trigger,request,SYM)
    
    
    database = "___task_"+trigger['name']+"_"+str(trigger['taskId']).zfill(3)
    trigger['n'] = int(trigger['n'])
    
    node = workflow[trigger['n']]
    nodeApp = list(node.keys())[0].replace("$","")
    
    #nodeRequest = SYM.mongo[database][last_collection].find({})
    collection = str(trigger['n']).zfill(3)+"_"+nodeApp
    
    if trigger['n'] > 0:
        
        last_node = workflow[trigger['n'] - 1]
        last_nodeApp = list(last_node.keys())[0].replace("$","")
        last_collection = str(trigger['n']-1).zfill(3)+"_"+last_nodeApp
        if last_nodeApp == "readCollection":
            last_query = SYM.mongo[SYM.db+"_"+last_node["$"+last_nodeApp]['database']][last_node["$"+last_nodeApp]['collection']]
        else:
            last_query = SYM.mongo[database][last_collection]
        
        result = runNode(database,nodeApp,node,last_query,trigger,workflow,request,SYM)
        saveResult(result,database,collection,SYM)
        current_query = SYM.mongo[database][collection]
        
    else:
        result=[]
    if nodeApp == "readCollection":
        current_query = SYM.mongo[SYM.db+"_"+node["$"+nodeApp]['database']][node["$"+nodeApp]['collection']]
    else:
        current_query = SYM.mongo[database][collection]
    
        
    
    trigger['n'] += 1
    trigger['update'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    SYM.mongo[database]['000_trigger'].update_many({}, { "$set": trigger })

    ret= nextStep(node,current_query,trigger,workflow,database,collection,result,request,SYM)
    return(ret)


import pymongo
from pymongo import MongoClient
import certifi
import json
import requests
import os


class symClient:
        def __init__(self, mongo, url,db,ssl = None):
                if ssl is not None:
                        self.env = "prod"
                        self.mongo = MongoClient(mongo, tlsCAFile=certifi.where())
                else:
                         self.env = "local"
                         self.mongo = MongoClient(mongo)
                self.url = url
                self.db = db

        def app(self,controller):
                core = __import__("app."+controller)
                file = getattr(core, controller)
                
                return(file)

        def workflow(self,controller):
                core = __import__("workflow."+controller)
                file = getattr(core, controller)
                
                return(file)

        def controller(self,controller):
                
                
                core = __import__("controller."+controller)
                file = getattr(core, controller)
                
                return(file)

        def cli(self,request):
                request.pop(0)
                #self.e.pop(0)
                try:
                        controller = request.pop(0)
                except:
                        return ("""Usage: sym [controleur] [--GET_KEY=GET_VALUE] [--data=DATA] [--dataFolder=DATA_FOLDER] [--dataUrl=DATA_URL]] [--help]""")
                request = list(request)
                core = __import__("controller."+controller)
                file = getattr(core, controller)
                Def = getattr(file, "run")
                
                get = {}
                folder = []
                body = {}
                
                for eItem in request:
                        if eItem[:2] == "--":
                                if eItem[2:] == "help":
                                        help = getattr(file, "help")
                                        return help()
                                else:
                                        splitItem = eItem[2:].split("=")
                                        if len(splitItem) == 1:
                                                get[splitItem[0]] = 1
                                        else:
                                                get[splitItem[0]] = splitItem[1]
                        else:
                                folder.append(eItem)
                
                if get.get("data") is not None:
                        body = json.loads(str(get["data"]))
                        #get.pop("data")
                        del get["data"]
                if get.get("dataFolder") is not None:
                        file = open(get['dataFolder'], 'r')
                        body = json.loads(file.read())
                        #get.pop("data")
                        del get["dataFolder"]
                if get.get("dataUrl") is not None:
                        response = requests.request("GET", get['dataUrl'])
                        body = response.json()
                        #get.pop("data")
                        del get["dataUrl"]
                request = {
                        "folder" : folder,
                        "get" : get,
                        "body" : body
                }
                return(Def(request,self))


      
        def http(self,request):
                #return(self.e.view_args)
                #return(self.e.get_json())
                #self.e.pop(0)
                #self.e.pop(0)
                #
                get = {}
                try:
                        path = request.view_args['path'].split("/")
                except:
                        path = []
                try:
                        body = request.get_json()
                except:
                        body = {}
                try:
                        for args in  request.args:
                                get[args] = request.args.get(args)
                except:
                        get = {}

                controller = path.pop(0)
                core = __import__("controller."+controller)
                file = getattr(core, controller)
                Def = getattr(file, "run")

                request = {
                        "folder" : path,
                        "get" : get,
                        "body" : body
                }
                return(Def(request,self))
        
        def pubSub(self,request):
                get = {}
                try:
                        path = request['folder']
                except:
                        path = []
                try:
                        body = request['body']
                except:
                        body = {}
                try:
                        get = request['get']
                except:
                        get = {}

                controller = path.pop(0)
                core = __import__("controller."+controller)
                file = getattr(core, controller)
                Def = getattr(file, "run")

                request = {
                        "folder" : path,
                        "get" : get,
                        "body" : body
                }
                return(Def(request,self))




        


import os
import requests
import json 

def help():
    return """source=SOURCE_NAME 
    body : {
        "subTime":1580000000
    }

    return : {
         "subTime":1580000000,
         "next" : 100 
    }
    dest=DEST_NAME 
    body : {
    "request": {
            "id_s": {
                [ID_S]
            },
            "properties": {
                [PROPERTIES]
            },
        }
    }
    return : {
        "id_s": {
            [ID_S]
        }
    }
    """

def run(request,SYM):
    n = 0
    getUrl = ""
    for get in request['get']:
        if n == 0:
            getUrl = getUrl + "?"+get+"="+request['get'][get]
        else:
            getUrl = getUrl + "&"+get+"="+request['get'][get]
        n += 1
    url = os.getenv('jsSYM_URL')+getUrl

    payload = json.dumps(request['body'])

    headers = {
    'Content-Type': 'application/json'
    }

    response = requests.request("POST", url, headers=headers, data=payload)

    return(response.json())



import os
def run(request,SYM):
    os.system('gcloud functions deploy pySYM_S --trigger-http --runtime python310 --allow-unauthenticated --region europe-west1 --entry-point=http')
    os.system('gcloud functions deploy pySYM_pubsub --trigger-topic="sym" --runtime python310 --allow-unauthenticated --region europe-west1 --entry-point=pubSub')


def trigger(request,SYM):
    return {
            "name": "workflowTest",
            "async": 0,
            #"result": "1",
            "save": "1",
            "#task": "1",
            "#backgroud": "1",
            "timeout": 3600,
            "limit": 60,
            "thread" : 1,
            #"shedule" : 3600
    }

def SYM(request,SYM):
    return [
        {
            "$py": {"$find" :{}}
        }
        # {
        #     "$test": {
        #         "dataTest": []
        #     }
        # }
    ]
    
def find(request,SYM):
    find = list(request['body']['nodes'][1]['request'].find({}))
    if len(find) > 0:
        find = find[0]
    else:
        find = {"$match": {}}
    if find.get('$limit') is None:
        find['$limit'] = 100
    if find.get('$match') is None:
        find['$match'] = {}

    if find.get('$aggregate') is None:
        find = list(request['body']['node']['request'].find(find['$match']).limit(find['$limit']))
    else:
        find = list(request['body']['node']['request'].aggregate(find['$aggregate']))
    return find


def trigger(request,SYM):
    return {
            "name": "workflowTest",
            "async": 0,
            #"result": "1",
            "save": "1",
            "#task": "1",
            "#backgroud": "1",
            "timeout": 3600,
            "limit": 60,
            "thread" : 1,
            "shedule" : 3600
        
    }

def SYM(request,SYM):
    return [
        {
            "$js": {
                "source": "get-clients-erp"
            }
        },
        {
            "$py": {"$test" : "test"}
        }
        # {
        #     "$test": {
        #         "dataTest": []
        #     }
        # }
    ]

def test(request,SYM):
    result = list(request['body']['node']['request'].find({}))
    result = result[0]['datas']
    return result


import base64

import requests
import json
import pymongo
from pymongo import MongoClient
from pymongo import UpdateOne, DeleteOne
from google.cloud import pubsub_v1
from google.oauth2 import service_account
import requests
import json
import traceback
from bson import json_util
import datetime
from datetime import  date, timedelta
import pytz

def valArray(val,Array):
    try:
        return Array[val]
    except:
        return ""



def findValueArray(sets,find):
    dEx = find.split("$.")

    find = find.split(".")
    val = sets
    ret = []
    dol = 0
    for f in find:
        if f == "$":
            dol = 1
            ret = []
            for v in val:
                if len(dEx) == 1:
                    ret.append(v)
                else:
                    #print(dEx[1])
                    #die()
                    ret.append(findValueArray(v,dEx[1])[0])
        if dol == 0:
            if f.isnumeric() == True:
                f = int(f)
            val = valArray(f,val)
            ret = [val]
    #print(ret)
    return ret

def mappage(request,mapp):
    returnMapp = {}
    for c in mapp:
        try:
            replace = mapp[c]["$replace"]
            mapp[c] = mapp[c]["$path"]
        except:
            replace = ""

        if mapp[c][0] == "$":
            fva = findValueArray(request,mapp[c][1: ])
            if len(fva) > 1:
                returnMapp[c] = fva
            else:
                returnMapp[c] = fva[0] if fva[0] != "" else replace
                
        else:
            returnMapp[c] = mapp[c]
    return(returnMapp)

def update(request,mapped,prefix):
    update = 0

    for r in mapped:
        try:
            if mapped[r] != request['lookup']['propertiesLookup']["["+prefix+"]"+r]:
                update = 1
        except:
            update = 1
    return(update)
def updateLookup(request,reponse,properties):
    print("updateLookup")
    paris = pytz.timezone('Europe/Paris')
    request['properties']['updateTime'] = paris.localize(datetime.datetime.utcnow())+ timedelta(hours=2)
    #request["prefix"] = properties['update']
    id_s = {}
    insertId = [ "task",properties['name'],str(request['properties']['updateTime'])]
    if reponse.get("error"):
        request['properties']["error"] = reponse["error"]
        
    else:
        request['properties']["error"] = "Any"
        
        for ids in reponse:
            if ids != "insertId":
                properties["request"]['id_s'][ids] = reponse[ids]

    for ids in properties["request"]['id_s']:
        if ids != "insertId":
            id_s[ids] = "id_s_"+ids
            request['properties']["id_s_"+ids] = properties["request"]['id_s'][ids]
            insertId.append(ids)
            insertId.append(properties["request"]['id_s'][ids])
        
    
    return {
        "LookUpName": properties["LookUpName"],##properties["LookUpName"]
        "datas": [request['properties']],
        "events": [
            {
                "eventName": "_Task_"+properties["action"]+"_"+properties['name'],
                "eventTime": "updateTime",
                "insertId": insertId
            }
        ],
        "Lookups": [
            {
                "value": "id_s_"+list(id_s.keys())[0],
                "lookup": properties["LookUpName"],
                "id": list(id_s.keys())[0],
                "key": properties["LookUpName"]
            }
        ],
        "ids": id_s,
        "prefix": "_Task_"+properties['name'],
        "properties": "",
        "db": "",
        "source" : ""
    }
def runTask(e):
    retRep = []
    print("startTask")
    url = e["properties"]['url']

    if e["properties"].get("subTime") is None:
        e["properties"]['subTime'] = None
    if e["properties"].get("next") is None:
        e["properties"]['next'] = None
    if e["properties"].get("query") is None:
        e["properties"]['query'] = []
    if e["properties"].get("request") is None:
        e["properties"]['request'] = []
    if e["properties"].get("mapp") is None:
        e["properties"]['mapp'] = {}
    if e["properties"].get("limit") is None:
        e["properties"]['limit'] = 100
    if e["properties"].get("force") is None:
        e["properties"]['limit'] = False
    if e["properties"].get("operation") is None:
        e["properties"]['operation'] = "insert"
    
    requestPayload = {
                "id_s" : {},
                "properties" : {},
                "update" : 1 
            }
    
    if len(e["properties"]['request']) > 0:
        
        
        mapped = mappage(e["properties"]['request'],e["properties"]['mapp'])
        
        if e["properties"]['action'] == "update":
            requestPayload['update'] = update(e["properties"]['request'],mapped,"_Task_"+e["properties"]['name']) 
        requestPayload['properties'] = mapped
        

        for id_s in e["properties"]['request']['id_s']:
            requestPayload['id_s'][id_s] = e["properties"]['request']['id_s'][id_s]
        
    #return(requestPayload)
    payload = json.dumps({
        "subTime": e["properties"]["subTime"],
        "next": e["properties"]["next"],
        "db": e["db"],
        "request" :requestPayload,
        "query" : e["properties"]["query"]
    })
    headers = {
        'Content-Type': 'application/json'
    }

    if requestPayload['update'] == 1:
        
        response = requests.request("POST", url, headers=headers, data=payload)
   
        
        try:
        #print(response.text)
            request = response.json()
        except:
            request = {"error":response.text}
    else:
        request = {"update" : 0}
    if len(requestPayload['properties']) > 0:
        request = updateLookup(requestPayload,request,e["properties"])
    # except:
    #     print("error bizare")
    #     request = {}
    credentials = service_account.Credentials.from_service_account_file(
                './file.json')
        
    publisher = pubsub_v1.PublisherClient(credentials=credentials)
    topic_path = publisher.topic_path("scrapyomama-335508", "trck")
    print("url")  
    print(url)
    print("payload request")
    print(payload) 
    print("reponse_get")
    print(request)  
    request['db'] = e['db']
    request['force'] = e['properties']['force']
    request['source'] = e['properties']['name']
    
    if request.get("datas") is not None:
        #if response.json().get('next') is not None:
        request['compil'] = 1
        retRep.append(json.loads(json.dumps(request, default=str)))
        data = json.dumps(request, default=str)
        data = data.encode("utf-8")
        print(data)
        future = publisher.publish(topic_path, data=data)
        print(future.result())
        print("import.")
        request['datas'] = [request['datas'][0]]
        print("senddata")
        

    rep = {
        "request" : e,
        "message" : "ok",
        "reponse" : request
    }
    
    data = {  
            "name" : "Trigguer",
            "lookup" : "monitor",    
            "properties" :rep,  
            "source" : request['source'] ,
            "db" : request['db'],
            "logs" : 0
            }
    

    if request.get('next') is not None:
        if e["properties"]['limit'] > 0:
            publisher = pubsub_v1.PublisherClient(credentials=credentials)
            topic_path = publisher.topic_path("scrapyomama-335508", "task")
            e["properties"]['limit'] = e["properties"]['limit'] - 1
            e["properties"]['next'] = response.json()["next"]
            print("sendNext")
            print(data) 
            #retRep.append(json.loads(json.dumps(e, default=str)))
            data = json.dumps(e, default=str)
            data = data.encode("utf-8")
            future = publisher.publish(topic_path, data=data)
            print(future.result())
            print("task.")
        
    #sendRunTaskLogs
    return retRep  
    
def main(e):

    
    request_json = e['body']
    try:
      return runTask(request_json)  
    except Exception as e:
      print(str(traceback.format_exc()))
      credentials = service_account.Credentials.from_service_account_file(
          './file.json')

      publisher = pubsub_v1.PublisherClient(credentials=credentials)
      topic_path = publisher.topic_path("scrapyomama-335508", "trck")

      
      rep = {
          "request" : request_json,
          "message" : "error",
          "reponse" : str(traceback.format_exc())
      }
      data = json.dumps({
            "event" : 
            {  
              "name" : "Compil",
              "lookup" : "monitor",    
              "properties" :rep,  
              "source" : request_json['source'] ,
              "db" : request_json['db'],
              "logs" : 0
            }
          })
      # Data must be a bytestring
      data = data.encode("utf-8")
      # When you publish a message, the client returns a future.
      future = publisher.publish(topic_path, data=data)
      return(str(traceback.format_exc()))

